#!/usr/bin/wish
#
# this code was modified from the original, at NeoSoft:
############################################################################
# Tcl/Tk analog clock 4							   #
#									   #
# by Karl Lehenbauer (karl@neosoft.com)					   #
#									   #
# Copyright (C) 1996 NeoSoft, All Rights Reserved			   #
#									   #
# Freely redistributable as long as copyright is not removed, we are not   #
# sued for any failure (even if we knew there was a problem and didn't     #
# tell you about it), and that proper credit is given to Karl Lehenbauer   #
# and NeoSoft, Inc, on any webpages incorporating this or derivitave code, #
# including a link to http://www.neosoft.com/neosoft/staff/karl		   #
#									   #
# Has hour, minute and second hand.					   #
#									   #
# Has selectable digital time formats.					   #
############################################################################

set pi 3.14159265358979

proc update_analog_time {clockFrame} {
    global second after1 after2

    set clock [clock seconds]
    scan [clock format [clock seconds] -format "%I %M %S"] "%d %d %d" hour minute second
#    $clockFrame itemconfigure hour -extent [expr -($hour * 30 + $minute / 2)]
#    $clockFrame itemconfigure minute -extent [expr -($minute * 6)]
    $clockFrame itemconfigure hour -start [expr (90 - ($hour*30+$minute/2) -2)]
    $clockFrame itemconfigure minute -start [expr (90 - ($minute * 6) -3)]
    for {set i 0} {$i < 12} {incr i} {
#        if [info exists after1] [after cancel $after1]
        set after1 [after [expr $i * 100] $clockFrame itemconfigure pip$i -fill black -outline white]
#        if [info exists after2] [after cancel $after2]
        set after2 [after [expr 1200 + $i * 100] $clockFrame itemconfigure pip$i -fill grey -outline black]
    }
}

proc update_seconds {clockFrame} {
    global second after3 tcl_platform

switch $tcl_platform(platform) unix { balloon_help $clockFrame [exec date] }
    if [info exists after3] [after cancel $after3]
    set after3 [after 1000 update_seconds $clockFrame]
    if {$second > 59} {
        update_analog_time $clockFrame
    }
    $clockFrame itemconfigure second -extent [expr -$second * 6]

    if {$second % 5 == 0} {
	$clockFrame itemconfigure pip[expr $second / 5] -fill white
    }
    incr second
}

# i have changed the colors and sizes, and (will) previne restart...
proc analog_clock {nframe} {
    global width height color

    canvas $nframe -width 60 -height 60 -background $color(back) \
		-borderwidth 0 -highlightthickness 0
    $nframe create arc   1  1 59 59 -width 0 -tag second -start 90 \
		-fill $color(high)
    $nframe create oval  6  6 54 54 -width 0 -fill $color(back) -outline ""
    draw_pips $nframe   30 30 25
    $nframe create arc  11 11 49 49 -width 0 -tag minute -extent 4 \
		-fill $color(text)
#    $nframe create oval 16 16 44 44 -width 0 -fill $color(back) -outline ""
    $nframe create arc  19 19 41 41 -width 0 -tag hour -extent 6 \
		-fill $color(text)
#    $nframe create oval 24 24 36 36 -width 0 -fill $color(back) -outline ""
#    $nframe create text 32 45 -text TkPanel -font { times 11 } -fill $color
    update_analog_time $nframe
    update_seconds $nframe
    return $nframe
}

proc draw_pip {canvas hour centerX centerY distance size} {
    global pi
    set x [expr $centerX + sin((6 - $hour) / 6.0 * $pi) * $distance]
    set y [expr $centerY + cos((6 - $hour) / 6.0 * $pi) * $distance]
    $canvas create oval [expr $x - $size] [expr $y - $size] [expr $x + $size] [expr $y + $size] -fill grey -tag "pips pip$hour" -width 0
}

proc draw_pips {canvas centerX centerY distance} {
    draw_pip $canvas 0 $centerX $centerY $distance 3
    for {set i 0} {$i < 12} {incr i} {
	draw_pip $canvas $i $centerX $centerY $distance 2
    }
}

proc update_digital_time {} {
    global digitalClockFormat digitalClockFrame

    $digitalClockFrame configure -text [clock format [clock seconds] -format $digitalClockFormat]
}

proc update_digital_time_and_repeat {} {
    global after4
    update_digital_time
    if [info exists after4] [after cancel $after4]
    set after4 [after [expr {60 - ([clock seconds] % 60)}]000 update_digital_time_and_repeat]
}

# changed the order of formats, and make some invisibles (their too large
# for a button)
proc set_digital_format {format} {
    global digitalClockFormat currentFormatNumber nFormats

    set nFormats 3

    case $format {
	0 {set digitalClockFormat "%A"}
        1 {set digitalClockFormat "%H:%M"}
        2 {set digitalClockFormat "%I:%M %p"}
# 3 and 4 are not used because their size
	3 {set digitalClockFormat "%d-%b-%y"}
	4 {set digitalClockFormat "%A, %B %d, %Y"}
    }

    set currentFormatNumber $format
}

proc next_format {} {
    global currentFormatNumber nFormats

    incr currentFormatNumber
    if {$currentFormatNumber >= $nFormats} {
	set currentFormatNumber 0
    }
    set_digital_format  $currentFormatNumber
    update_digital_time
}

proc digital_clock {nframe} {
    global digitalClockFrame color

    set digitalClockFrame $nframe
    button $nframe -font { times 10 } -borderwidth 0 -command next_format \
		-background $color(back) -foreground $color(text) \
		-padx 0 -pady 0 -highlightthickness 0

    update_digital_time_and_repeat
    return
}

set_digital_format 0

# this one is the 'gate' between the panel and the clock
proc panel_clock { base } {
#global updated after1 after2 after3 after4
#if {$updated} {
#	puts "updated."
#	after cancel $after1
#	after cancel $after2
#	after cancel $after3
#	after cancel $after4
#}
    frame $base.clock
    digital_clock $base.clock.dclock
    analog_clock $base.clock.aclock
    pack $base.clock.dclock -fill x -side top
    pack $base.clock.aclock -fill both -expand 1
pack $base.clock -side left
}

