# parses the .tkpanelrc file, to extracts global definitions, buttons and
# subpanels
proc read_rc { rcfile } {
	global pager_names icons_path lock_cmd color
	if [ file exists $rcfile ] {
		set fileID [ open $rcfile r ]
		while { [ gets $fileID line ] >=0 } {
			if { [ string index $line 0 ] == "#" } {
				# it's a comment like this!
			} elseif [ regexp "pager (.*)" $line all value ] {
				set pager_names(1) [lindex $value 0]
				set pager_names(2) [lindex $value 1]
				set pager_names(3) [lindex $value 2]
				set pager_names(4) [lindex $value 3]
			} elseif [ regexp "icons_path (.*)" $line all value ] {
				set icons_path $value
			} elseif [ regexp "lock_cmd (.*)" $line all value ] {
				set lock_cmd $value
			} elseif [ regexp "button (.*)" $line all value ] {
				get_button $fileID $value
			} elseif [ regexp "colors (.*)" $line all value ] {
				set color(back) [lindex $value 0]
				set color(fore) [lindex $value 1]
				set color(high) [lindex $value 2]
				set color(text) [lindex $value 3]
			} elseif [ regexp "subpanel (.*)" $line all value ] {
				get_subpanel $fileID $value
			}
		}
	}
}

# parses the button section; gets its side, command and related icon
proc get_button { fileID name } {
	global lbuttons rbuttons
	gets $fileID line
	while { ![ string match "end" $line ] } {
		if [ regexp "place (.*)" $line all value ] {
			set place $value
		} elseif [ regexp "command (.*)" $line all value ] {
			set command $value
		} elseif  [ regexp "icon (.*)" $line all value ] {
			set icon $value
		}
		if [ info exists icon ] "store_icon $icon"
		gets $fileID line
	}
	if [ string match "left" $place ] {
		lappend lbuttons "\"$name\" \"$command\" \"$icon\""
	} elseif [ string match "right" $place ] {
		lappend rbuttons  "\"$name\" \"$command\" \"$icon\""
	}
}

# parses the subpanel section
proc get_subpanel { fileID name } {
	global subpanel
	gets $fileID line
	while { ![ string match "end" $line ] } {
		if [ regexp "label (.*)" $line all value ] {
			set cmd_label $value
		} elseif [ regexp "command (.*)" $line all value ] {
			set cmd_command $value
		} elseif  [ regexp "icon (.*)" $line all value ] {
			set cmd_icon $value
		}
		if [info exists cmd_icon ] "store_icon $cmd_icon"
		gets $fileID line
	}
	lappend subpanel($name) "\"$cmd_label\" \"$cmd_command\" \"$cmd_icon\""
}

